@@ -4,7 +4,7 @@
  * Start Date:  Thu Dec 28 13:34:47 1995
  * Project:     INN (innfeed)
  * File:        connection.c
- * RCSId:       $Id: connection.c,v 1.20 1996/05/23 22:40:32 brister Exp $
+ * RCSId:       $Id: connection.c,v 1.21 1996/06/05 14:00:48 brister Exp $
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
  *              Permission to use, copy, modify, and distribute this
@@ -78,7 +78,7 @@
   */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: connection.c,v 1.20 1996/05/23 22:40:32 brister Exp $" ;
+static const char *rcsid = "$Id: connection.c,v 1.21 1996/06/05 14:00:48 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -121,6 +121,12 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include "article.h"
 #include "msgs.h"
 
+#if defined (NDEBUG)
+#define VALIDATE_CONNECTION(x) ((void) 0)
+#else
+#define VALIDATE_CONNECTION(x) validateConnection (x)
+#endif
+
 /*
  * Private types.
  */
@@ -164,6 +170,7 @@ struct connection_s
     ArtHolder takeRespHead ;    /* list of articles waiting on
                                    TAKETHIS/IHAVE-body response */
     u_int articleQTotal ;       /* number of articles in all four queues */
+    ArtHolder missing ;         /* head of missing list */
 
 
     Buffer respBuffer ;         /* buffer all responses are read into */
@@ -273,6 +280,7 @@ static void processResponse400 (Connection cxn, char *response) ;
 static void processResponse435 (Connection cxn, char *response) ;
 static void processResponse436 (Connection cxn, char *response) ;
 static void processResponse437 (Connection cxn, char *response) ;
+static void processResponse480 (Connection cxn, char *response) ;
 
 
 /* Misc functions */
@@ -401,6 +409,7 @@ Connection newConnection (Host host,
   cxn->offThreshold = lowPassLow ;
 
   cxn->sleepTimerId = 0 ;
+  cxn->sleepTimeout = INITIAL_REESTABLISHMENT_PERIOD ;
 
   resetConnection (cxn) ;
 
@@ -557,8 +566,9 @@ void cxnWait (Connection cxn)
   ASSERT (cxn->state == cxnStartingS ||
           cxn->state == cxnSleepingS ||
           cxn->state == cxnConnectingS ||
+          cxn->state == cxnFeedingS ||
           cxn->state == cxnFlushingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   abortConnection (cxn) ;
 
@@ -587,7 +597,7 @@ void cxnFlush (Connection cxn)
   ASSERT (cxn != NULL) ;
   ASSERT (cxn->state != cxnStartingS) ;
   ASSERT (cxn->state != cxnDeadS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   switch (cxn->state)
     {
@@ -646,7 +656,7 @@ void cxnClose (Connection cxn)
   ASSERT (cxn != NULL) ;
   ASSERT (cxn->state != cxnDeadS) ;
   ASSERT (cxn->state != cxnStartingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   switch (cxn->state)
     {
@@ -686,7 +696,7 @@ void cxnClose (Connection cxn)
         die ("Bad connection state: %s\n",stateToString (cxn->state)) ;
     }
 
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   if (cxn->state == cxnDeadS)
     {
@@ -712,7 +722,7 @@ bool cxnTakeArticle (Connection cxn, Article art)
   bool rval = true ;
 
   ASSERT (cxn != NULL) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   if ( !cxnQueueArticle (cxn,art) ) /* doesn't change the Connection state */
     return false ;
@@ -722,7 +732,7 @@ bool cxnTakeArticle (Connection cxn, Article art)
           cxn->state == cxnWaitingS) ;
 
   if (cxn->state != cxnWaitingS) /* because articleQTotal == 1 */
-    validateConnection (cxn) ;
+    VALIDATE_CONNECTION (cxn) ;
   else
     ASSERT (cxn->articleQTotal == 1) ;
 
@@ -769,7 +779,7 @@ bool cxnQueueArticle (Connection cxn, Article art)
   ASSERT (cxn != NULL) ;
   ASSERT (cxn->state != cxnStartingS) ;
   ASSERT (cxn->state != cxnDeadS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   switch (cxn->state)
     {
@@ -1109,7 +1119,7 @@ static void connectionDone (EndPoint e, IoStatus i, Buffer *b, void *d)
              tried the MODE STREAM command. */
         }
     }
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 }
 
 
@@ -1261,7 +1271,7 @@ static void getModeResponse (EndPoint e, IoStatus i, Buffer *b, void *d)
   ASSERT (b [1] == NULL) ;      /* only ever one buffer on this read */
   ASSERT (cxn->state == cxnConnectingS) ;
   ASSERT (!writeIsPending (cxn->myEp)) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   peerName = hostPeerName (cxn->myHost) ;
 
@@ -1302,47 +1312,48 @@ static void getModeResponse (EndPoint e, IoStatus i, Buffer *b, void *d)
   else
     {
       syslog (LOG_NOTICE,CONNECTED,peerName, cxn->ident) ;
-
-      /* now we consider ourselves completly connected. */
-      cxn->timeCon = theTime () ;
-      cxn->state = cxnFeedingS ;
-
+          
       switch (code)
         {
-          case 203:                     /* will do streaming */
+          case 203:             /* will do streaming */
             hostRemoteStreams (cxn->myHost, cxn, true) ;
             cxn->doesStreaming = true ;
             cxn->maxCheck = hostMaxChecks (cxn->myHost) ;
             break ;
-
+                
           default:                      /* won't do it */
             hostRemoteStreams (cxn->myHost, cxn, false) ;
             cxn->maxCheck = 1 ;
             break ;
         }
-
-      /* one for the connection and one for the buffer array */
+          
+      /* now we consider ourselves completly connected. */
+      cxn->timeCon = theTime () ;
+      cxn->state = cxnFeedingS ;
+          
+          /* one for the connection and one for the buffer array */
       ASSERT (bufferRefCount (cxn->respBuffer) == 2) ;
-
+          
       /* there was only one line in there, right? */
       bufferSetDataSize (cxn->respBuffer, 0) ;
       buffers = makeBufferArray (bufferTakeRef (cxn->respBuffer), NULL) ;
-
-      /* sleepTimeout get changed at each fialed attempt, so reset. */
+          
+          /* sleepTimeout get changed at each failed attempt, so reset. */
       cxn->sleepTimeout = INITIAL_REESTABLISHMENT_PERIOD ;
-
+          
       if ( !prepareRead (cxn->myEp, buffers, responseIsRead, cxn, 1) )
         {
           freeBufferArray (buffers) ;
-
+              
           cxnSleep (cxn) ;
         }
       else
         {
           if (cxn->articleReceiptTimeout > 0)
             cxn->artReceiptTimerId = prepareSleep (articleTimeoutCbk,
-                                                   cxn->articleReceiptTimeout, cxn) ;
-
+                                                   cxn->articleReceiptTimeout,
+                                                   cxn) ;
+              
           /* now we wait for articles from our Host, or we have some
              articles already. On infrequently used connections, the
              network link is torn down and rebuilt as needed. So we may
@@ -1352,7 +1363,7 @@ static void getModeResponse (EndPoint e, IoStatus i, Buffer *b, void *d)
             doSomeWrites (cxn) ;
         }
     }
-
+  
   freeBufferArray (b) ;
 }
 
@@ -1384,7 +1395,7 @@ static void responseIsRead (EndPoint e, IoStatus i, Buffer *b, void *d)
   ASSERT (cxn->state == cxnFeedingS ||
           cxn->state == cxnClosingS ||
           cxn->state == cxnFlushingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   bufferAddNullByte (b [0]) ;
 
@@ -1536,19 +1547,26 @@ static void responseIsRead (EndPoint e, IoStatus i, Buffer *b, void *d)
             processResponse400 (cxn, response) ;
             break ;
 
+            
 
           case 235:             /* article transfered OK (IHAVE-body) */
             processResponse235 (cxn, response) ;
             break ;
 
 
+          case 480:             /* Transfer permission denied. */
+            processResponse480  (cxn,response) ;
+            break ;
+            
+
           default:
             syslog (LOG_ERR, UNKNOWN_RESPONSE, peerName, cxn->ident,
                     code, response) ;
+            cxnSleep (cxn) ;
             break ;
         }
 
-      validateConnection (cxn) ;
+      VALIDATE_CONNECTION (cxn) ;
 
       if (cxn->state != cxnFeedingS && cxn->state != cxnClosingS &&
           cxn->state != cxnFlushingS)
@@ -1618,26 +1636,25 @@ static void responseIsRead (EndPoint e, IoStatus i, Buffer *b, void *d)
             if (cxn->checkRespHead != NULL || cxn->takeRespHead != NULL)
               initReadBlockedTimeout (cxn) ;
 
-            validateConnection (cxn) ;
+            VALIDATE_CONNECTION (cxn) ;
 
             dprintf (5,"%s:%d about to do some writes\n",
                      hostPeerName (cxn->myHost),cxn->ident) ;
 
             doSomeWrites (cxn) ;
           }
-        validateConnection (cxn) ;
+        VALIDATE_CONNECTION (cxn) ;
         break ;
 
-      case cxnWaitingS:       /* presumably after a code 205 */
-      case cxnConnectingS:
-        ASSERT (code == 205) ;
+      case cxnWaitingS:         /* presumably after a code 205 or 400 */
+      case cxnConnectingS:      /* presumably after a code 205 or 400 */
+      case cxnSleepingS:        /* probably after a 480 */
         break ;
 
       case cxnDeadS:
         delConnection (cxn) ;
         break ;
 
-      case cxnSleepingS:
       case cxnStartingS:
       default:
         die ("Bad connection state: %s\n",stateToString (cxn->state)) ;
@@ -1659,7 +1676,7 @@ static void quitWritten (EndPoint e, IoStatus i, Buffer *b, void *d)
   peerName = hostPeerName (cxn->myHost) ;
 
   ASSERT (cxn->myEp == e) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   if (i != IoDone)
     {
@@ -1795,7 +1812,7 @@ static void responseTimeoutCbk (TimeoutId id, void *data)
           cxn->state == cxnFeedingS ||
           cxn->state == cxnFlushingS ||
           cxn->state == cxnClosingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   peerName = hostPeerName (cxn->myHost) ;
 
@@ -1848,7 +1865,7 @@ static void flushCxnCbk (TimeoutId id, void *data)
   ASSERT (cxn->state == cxnFeedingS ||
           cxn->state == cxnConnectingS) ;
   ASSERT (id == cxn->flushTimerId) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   cxn->flushTimerId = 0 ;
 
@@ -1876,7 +1893,7 @@ static void articleTimeoutCbk (TimeoutId id, void *data)
 
   ASSERT (cxn->state == cxnFeedingS) ;
   ASSERT (cxn->artReceiptTimerId == id) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   cxn->artReceiptTimerId = 0 ;
 
@@ -1954,7 +1971,7 @@ static void processResponse205 (Connection cxn, char *response)
 {
   bool immedRecon ;
 
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
   ASSERT (cxn->state == cxnFeedingS ||
           cxn->state == cxnFlushingS ||
           cxn->state == cxnClosingS) ;
@@ -2023,7 +2040,7 @@ static void processResponse238 (Connection cxn, char *response)
   ASSERT (cxn->state == cxnFlushingS ||
           cxn->state == cxnFeedingS ||
           cxn->state == cxnClosingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   msgid = getMsgId (response) ;
 
@@ -2059,7 +2076,7 @@ static void processResponse431 (Connection cxn, char *response)
   ASSERT (cxn->state == cxnFlushingS ||
           cxn->state == cxnFeedingS ||
           cxn->state == cxnClosingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   msgid = getMsgId (response) ;
 
@@ -2094,7 +2111,7 @@ static void processResponse438 (Connection cxn, char *response)
   ASSERT (cxn->state == cxnFlushingS ||
           cxn->state == cxnFeedingS ||
           cxn->state == cxnClosingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   msgid = getMsgId (response) ;
 
@@ -2130,7 +2147,7 @@ static void processResponse239 (Connection cxn, char *response)
   ASSERT (cxn->state == cxnFlushingS ||
           cxn->state == cxnFeedingS ||
           cxn->state == cxnClosingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   msgid = getMsgId (response) ;
 
@@ -2168,7 +2185,7 @@ static void processResponse439 (Connection cxn, char *response)
   ASSERT (cxn->state == cxnFlushingS ||
           cxn->state == cxnFeedingS ||
           cxn->state == cxnClosingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   msgid = getMsgId (response) ;
 
@@ -2208,7 +2225,7 @@ static void processResponse235 (Connection cxn, char *response)
           cxn->state == cxnClosingS) ;
   ASSERT (cxn->articleQTotal == 1) ;
   ASSERT (cxn->takeRespHead != NULL) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   /* now remove the article from the queue and tell the Host to forget
      about it. */
@@ -2239,7 +2256,7 @@ static void processResponse335 (Connection cxn, char *response)
           cxn->state == cxnClosingS) ;
   ASSERT (cxn->articleQTotal == 1) ;
   ASSERT (cxn->checkRespHead != NULL) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   /* now move the article into the third queue */
   cxn->takeHead = cxn->checkRespHead ;
@@ -2262,7 +2279,7 @@ static void processResponse400 (Connection cxn, char *response)
   ASSERT (cxn->state == cxnFlushingS ||
           cxn->state == cxnFeedingS ||
           cxn->state == cxnClosingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   /* We may get a response 400 multiple times when in streaming mode. */
   syslog (LOG_NOTICE,CXN_BLOCKED,hostPeerName(cxn->myHost),cxn->ident,
@@ -2295,7 +2312,7 @@ static void processResponse435 (Connection cxn, char *response)
           cxn->state == cxnClosingS) ;
   ASSERT (cxn->articleQTotal == 1) ;
   ASSERT (cxn->checkRespHead != NULL) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   cxn->articleQTotal-- ;
   cxn->checksRefused++ ;
@@ -2322,7 +2339,8 @@ static void processResponse435 (Connection cxn, char *response)
 
 
 /*
- * process the "transfer failed" response to the IHAVE-body.
+ * process the "transfer failed" response to the IHAVE-body, (seems this
+ * can come from the IHAVE too).
  */
 static void processResponse436 (Connection cxn, char *response)
 {
@@ -2335,14 +2353,22 @@ static void processResponse436 (Connection cxn, char *response)
           cxn->state == cxnFeedingS ||
           cxn->state == cxnClosingS) ;
   ASSERT (cxn->articleQTotal == 1) ;
-  ASSERT (cxn->takeRespHead != NULL) ;
-  validateConnection (cxn) ;
+  ASSERT (cxn->takeRespHead != NULL || cxn->checkRespHead != NULL) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   cxn->articleQTotal-- ;
 
-  artHolder = cxn->takeRespHead ;
-  cxn->takeRespHead = NULL ;
-
+  if (cxn->takeRespHead != NULL) /* IHAVE-body command barfed */
+    {
+      artHolder = cxn->takeRespHead ;
+      cxn->takeRespHead = NULL ;
+    }
+  else                          /* IHAVE command barfed */
+    {
+      artHolder = cxn->checkRespHead ;
+      cxn->checkRespHead = NULL ;
+    }
+  
   hostArticleDeferred (cxn->myHost, cxn, artHolder->article) ;
   delArtHolder (artHolder) ;
 }
@@ -2367,7 +2393,7 @@ static void processResponse437 (Connection cxn, char *response)
           cxn->state == cxnClosingS) ;
   ASSERT (cxn->articleQTotal == 1) ;
   ASSERT (cxn->takeRespHead != NULL) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   cxn->articleQTotal-- ;
   cxn->takesRejected++ ;
@@ -2380,6 +2406,31 @@ static void processResponse437 (Connection cxn, char *response)
 }
 
 
+/* Process the response 480 Transfer permission defined. We're probably
+   talking to a remote nnrpd on a system that forgot to put us in
+   the hosts.nntp */
+static void processResponse480 (Connection cxn, char *response)
+{
+  ArtHolder artHolder ;
+
+  (void) response ;             /* keep lint happy */
+
+  ASSERT (!cxn->doesStreaming) ;
+  ASSERT (cxn->state == cxnFlushingS ||
+          cxn->state == cxnFeedingS ||
+          cxn->state == cxnClosingS) ;
+  VALIDATE_CONNECTION (cxn) ;
+
+  syslog (LOG_NOTICE,NO_TRANSFER_NNRPD,hostPeerName (cxn->myHost),
+          cxn->ident) ;
+  
+  if (cxn->state == cxnClosingS)
+    cxnDead (cxn) ;
+  else
+    cxnSleep (cxn) ;
+}
+
+
 
 
 
@@ -2402,7 +2453,7 @@ static void cxnSleep (Connection cxn)
   ASSERT (cxn->state == cxnFlushingS ||
           cxn->state == cxnFeedingS ||
           cxn->state == cxnConnectingS) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   abortConnection (cxn) ;
 
@@ -2418,7 +2469,7 @@ static void cxnSleep (Connection cxn)
 static void cxnDead (Connection cxn)
 {
   ASSERT (cxn != NULL) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   abortConnection (cxn) ;
   cxn->state = cxnDeadS ;
@@ -2462,7 +2513,7 @@ static void noSuchMessageId (Connection cxn, u_int responseCode,
 static void abortConnection (Connection cxn)
 {
   ASSERT (cxn != NULL) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   /* cxn->myEp could be NULL if we get here during cxnConnect (via
      cxnWait()) */
@@ -2545,7 +2596,6 @@ static void resetConnection (Connection cxn)
 
   cxn->filterValue = 0.0 ;
 
-  cxn->sleepTimeout = INITIAL_REESTABLISHMENT_PERIOD ;
 }
 
 
@@ -2674,7 +2724,7 @@ static bool issueIHAVE (Connection cxn)
   ASSERT (cxn->articleQTotal == 1) ;
   ASSERT (cxn->checkHead != NULL) ;
   ASSERT (writeIsPending (cxn->myEp) == false) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   artH = cxn->checkHead ;
   article = cxn->checkHead->article ;
@@ -2715,6 +2765,9 @@ static bool issueIHAVE (Connection cxn)
           die ("Prepare write for IHAVE failed") ;
         }
 
+      /* setup timeout for read */
+      initReadBlockedTimeout (cxn) ;
+
       /* now move the article to the second queue */
       cxn->checkRespHead = cxn->checkHead ;
       cxn->checkHead = NULL ;
@@ -2748,7 +2801,7 @@ static void issueIHAVEBody (Connection cxn)
           cxn->state == cxnClosingS) ;
   ASSERT (cxn->articleQTotal == 1) ;
   ASSERT (cxn->takeHead != NULL) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   article = cxn->takeHead->article ;
   ASSERT (article != NULL) ;
@@ -2800,13 +2853,13 @@ static bool issueStreamingCommands (Connection cxn)
 {
   Buffer checkBuffer = NULL ;   /* the buffer with the CHECK commands in it. */
   Buffer *writeArray = NULL ;
-  ArtHolder p ;
+  ArtHolder p, q ;
   bool rval = false ;
 
   ASSERT (cxn != NULL) ;
   ASSERT (cxn->myEp != NULL) ;
   ASSERT (cxn->doesStreaming) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   checkBuffer = buildCheckBuffer (cxn) ; /* may be null if none to issue */
 
@@ -2838,6 +2891,9 @@ static bool issueStreamingCommands (Connection cxn)
           die ("Prepare write for STREAMING commands failed") ;
         }
 
+      /* setup timer for read timeout */
+      initReadBlockedTimeout (cxn) ;
+
       rval = true ;
 
       /* now shift articles over to their new queue. */
@@ -2852,6 +2908,18 @@ static bool issueStreamingCommands (Connection cxn)
       cxn->takeHead = NULL ;
     }
 
+  /* we defer the missing article notification to here because if there
+     was a big backlog of missing articles *and* we're running in
+     no-CHECK mode, then the Host would be putting bad articles on the
+     queue we're taking them off of. */
+  for (p = cxn->missing ; p != NULL ; p = q)
+    {
+      hostArticleIsMissing (cxn->myHost, cxn, p->article) ;
+      q = p->next ;
+      delArtHolder (p) ;
+    }
+  cxn->missing = NULL ;
+
   return rval ;
 }
 
@@ -2940,8 +3008,6 @@ static Buffer *buildTakethisBuffers (Connection cxn, Buffer checkBuffer)
   Buffer *rval = NULL ;
   const char *peerName = hostPeerName (cxn->myHost) ;
 
-  missing = NULL ;
-
   if (checkBuffer != NULL)
     lenArray++ ;
 
@@ -2991,8 +3057,8 @@ static Buffer *buildTakethisBuffers (Connection cxn, Buffer checkBuffer)
               ASSERT (cxn->articleQTotal > 0) ;
               cxn->articleQTotal-- ;
 
-              ta->next = missing ;
-              missing = ta ;
+              ta->next = cxn->missing ;
+              cxn->missing = ta ;
             }
           else
             {
@@ -3035,17 +3101,6 @@ static Buffer *buildTakethisBuffers (Connection cxn, Buffer checkBuffer)
             }
         }
 
-      /* we defer the missing article notification to here because if there
-         was a big backlog of missing articles *and* we're running in
-         no-CHECK mode, then the Host would be putting bad articles on the
-         queue we're taking them off of. */
-      for (p = missing ; p != NULL ; p = q)
-        {
-          hostArticleIsMissing (cxn->myHost, cxn, p->article) ;
-          q = p->next ;
-          delArtHolder (p) ;
-        }
-
       if (writeIdx > 0)
         rval [writeIdx] = NULL ;
       else
@@ -3075,7 +3130,7 @@ static void issueQUIT (Connection cxn)
 
   ASSERT (cxn->takeHead == NULL) ;
   ASSERT (cxn->checkHead == NULL) ;
-  validateConnection (cxn) ;
+  VALIDATE_CONNECTION (cxn) ;
 
   if (cxn->quitWasIssued)
     return ;
