@@ -4,7 +4,7 @@
  * Start Date:  Wed Nov 29 23:08:24 1995
  * Project:     INN (innfeed)
  * File:        endpoint.c
- * RCSId:       $Id: endpoint.c,v 1.10 1996/04/13 02:51:33 brister Exp $
+ * RCSId:       $Id: endpoint.c,v 1.12 1996/05/23 21:18:24 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -38,7 +38,7 @@
  */
 
 #if ! defined (lint)
-static char rcsid [] = "$Id: endpoint.c,v 1.10 1996/04/13 02:51:33 brister Exp $" ;
+static char rcsid [] = "$Id: endpoint.c,v 1.12 1996/05/23 21:18:24 brister Exp $" ;
 static void use_rcsid (const char *rId) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rId) ;
 } 
@@ -68,7 +68,7 @@ static void use_rcsid (const char *rId) {   /* Never called */
 #include <sys/uio.h>
 #include <sys/socket.h>
 
-#if DO_NEED_STREAM
+#if defined (DO_NEED_STREAM)
 #include <sys/stream.h>
 #endif
 
@@ -92,7 +92,7 @@ struct timeval
 
 #if ! defined (UIO_MAXIOV) && defined (MAX_IOVEC)
 #define UIO_MAXIOV MAX_IOVEC
-#elif !defined (UIO_MAXIOV) && defined (DEF_MAX_IOV) /* svr4 */
+#elif ! defined (UIO_MAXIOV) && defined (DEF_MAX_IOV) /* svr4 */
 #define UIO_MAXIOV DEF_MAX_IOV
 #endif
 
@@ -155,6 +155,9 @@ void (*gPrintInfo) (void) ;
 static int sigFlag ;
 static EndPoint mainEndPoint ;
 
+time_t  PrivateTime;
+
+
   /* private functions */
 static IoStatus doRead (EndPoint endp) ;
 static IoStatus doWrite (EndPoint endp) ;
@@ -303,10 +306,6 @@ void delEndPoint (EndPoint ep)
 
   endPointCount-- ;
 
-#ifdef DEADBEEF
-  deadBeef (ep, sizeof (struct endpoint_s)) ;
-#endif
-  
   FREE (ep) ;
 }
 
@@ -421,10 +420,10 @@ int prepareWrite (EndPoint endp,
 TimeoutId prepareWake (EndpTCB func, time_t timeToWake, void *clientData) 
 {
   TimeoutId id ;
-  time_t now = time (NULL) ;
+  time_t now = theTime() ;
   int timeToSleep ;
 
-  if (now < timeToWake)
+  if (now > timeToWake)
     return 0 ;
   
   timeToSleep = timeToWake - now ;
@@ -443,7 +442,7 @@ TimeoutId prepareWake (EndpTCB func, time_t timeToWake, void *clientData)
 /* queue up a new timeout request to off TIMETOSLEEP seconds from now */
 TimeoutId prepareSleep (EndpTCB func, int timeToSleep, void *clientData) 
 {
-  time_t now = time (NULL) ;
+  time_t now = theTime() ;
   TimeoutId id ;
   
   id = timerElemAdd (now + timeToSleep,func,clientData) ;
@@ -481,10 +480,6 @@ bool removeTimeout (TimeoutId tid)
   else
     p->next = n->next ;
 
-#ifdef DEADBEEF
-  deadBeef (n,sizeof (struct timerqelem_s)) ;
-#endif
-  
   FREE (n) ;
         
   return true ;
@@ -537,6 +532,8 @@ void Run ()
         
       sval = select (highestFd + 1, &rSet, &wSet, &eSet, twait) ;
 
+      timePasses () ;
+      
       if (sval == 0 && twait == NULL)
         die ("No fd's ready and no timeouts") ;
       else if (sval < 0 && errno == EINTR)
@@ -592,6 +589,8 @@ void Run ()
                           
                           if (ep->inCbk != NULL)
                             (*ep->inCbk) (ep,rval,buff,ep->inClientData) ;
+                          else
+                            freeBufferArray (buff) ;
                         }
                       else
                         {
@@ -620,6 +619,8 @@ void Run ()
                           
                           if (ep->outCbk != NULL)
                             (*ep->outCbk) (ep,rval,buff,ep->outClientData) ;
+                          else
+                            freeBufferArray (buff) ;
                         }
                       else
                         {
@@ -655,7 +656,15 @@ void Run ()
               
               if ( !FD_ISSET (fd,&rSet) && !FD_ISSET (fd,&wSet) )
                 if (ep->workCbk != NULL)
-                  ep->workCbk (ep,ep->workData) ;
+                  {
+                    EndpWorkCbk func = ep->workCbk ;
+                    void *data = ep->workData ;
+
+                    ep->workCbk = NULL ;
+                    ep->workData = NULL ;
+                    func (ep,data) ;
+                  }
+              
             }
         }
     }
@@ -820,10 +829,6 @@ static IoStatus doRead (EndPoint endp)
       else                      /* i < 0 && errno == EINTR */
         rval = IoIncomplete ;
 
-#ifdef DEADBEEF
-      deadBeef (vp, sizeof (struct iovec) * bCount) ;
-#endif
-      
       FREE (vp) ;
     }
   else
@@ -922,10 +927,6 @@ static IoStatus doWrite (EndPoint endp)
       else
         dprintf (1,"Wrote 0 bytes in doWrite()?\n") ;
 
-#ifdef DEADBEEF
-      deadBeef (vp, sizeof (struct iovec) * bCount) ;
-#endif
-      
       FREE (vp) ;
     }
   else
@@ -1088,7 +1089,7 @@ static struct timeval *getTimeout (struct timeval *tout)
   
   if (timeoutQueue != NULL)
     {
-      time_t now = time (NULL) ;
+      time_t now = theTime() ;
 
       while (timeoutQueue && now > timeoutQueue->when)
         doTimeout () ;
@@ -1124,10 +1125,6 @@ static void doTimeout (void)
 
   timeoutQueue = timeoutQueue->next ;
 
-#ifdef DEADBEEF
-  deadBeef (p,sizeof (struct timerqelem_s)) ;
-#endif
-  
   FREE (p) ;
   
   if (cbk)
@@ -1138,7 +1135,7 @@ static void doTimeout (void)
 
 
 
-#ifdef WANT_MAIN
+#if defined (WANT_MAIN)
 
 
 #include <netinet/in.h>
@@ -1235,7 +1232,7 @@ void lineIsRead (EndPoint myEp, IoStatus status, Buffer *buffer, void *d)
   prepareRead (myEp,readBuffers,lineIsRead,NULL,1) ;
 
 #if 0
-  myEp->registerWake (&timerCallback,time(NULL) + 7,0) ;
+  myEp->registerWake (&timerCallback,theTime() + 7,0) ;
 #endif
 }
 
@@ -1245,7 +1242,7 @@ static void printDate (TimeoutId tid, void *data)
 {
   time_t t ;
 
-  t = time (NULL) ;
+  t = theTime() ;
   
   dprintf (1,"Timeout (%d) time now is %ld %s",
            (int) tid,(long) t,ctime(&t)) ;
@@ -1266,7 +1263,7 @@ static void Timeout (TimeoutId tid, void *data)
   static int seeded ;
   static int howMany ;
   static int i ;
-  time_t t = time (NULL) ;
+  time_t t = theTime() ;
 
   if ( !seeded )
     {
@@ -1336,7 +1333,7 @@ int main (int argc, char **argv)
   struct sockaddr_in accNet ;
   int accFd = socket (AF_INET,SOCK_STREAM,0) ;
   u_short port = atoi (argc > 1 ? argv[1] : "10000") ;
-  time_t t = time (NULL) ;
+  time_t t = theTime() ;
 
 
   program = strrchr (argv[0],'/') ;
@@ -1369,7 +1366,7 @@ int main (int argc, char **argv)
 
   prepareSleep (Timeout,5,(void *) 0x10) ;
 
-  t = time (NULL) ;
+  t = theTime() ;
   dprintf (1,"Time now is %s",ctime(&t)) ;
   
   prepareWake (printDate,t + 16,NULL) ;
